(function($) {

    RAVEN.namespace("RAVEN.TestSet");

    /**
     * Move positions constants
     */
    var AFTER = "After";
    var BEFORE = "Before";

    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.TestSet.TestsViewModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testset/' + this._getTestSetKey() + '/columns';
        },

        defaults: function() {
            return [];
        },

        _getTestSetKey: function() {
            return $('#raven-testset-tests-issue-key').val();
        }
    });

    /**
     * This is the main view for the Tests table on a Test Set issue
     */
    RAVEN.TestSet.TestsView = Backbone.View.extend({
        el: '#raven-testset-tests',

        /**
         * DataTable Instance
         */
        dataTable: null,

        /**
         * The Test Set issue key
         */
        testSetKey: null,

        /**
         * Permissions
         */
        canEdit: false,

        /**
         * The Bulk view
         */
        bulkEditView: null,

        currentDropdownTrigger: null,

        events: {
            'click #raven-testset-tests #raven-add-tests': '_onAddTestsClick',
            'click #raven-testset-tests #raven-create-test': '_onCreateTestClick'
        },

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.testset a.raven-remove-command': '_onTestDeleteClick',
            'click.raven.testset .bulk-remove-command': '_onBulkTestDeleteClick',
            'click.raven.testset a.testset-raven-rank-change-command': '_onChangeTestRankClick',
            'click.raven.testset .testset-raven-dropdown-trigger': '_onDropDownTriggerClick',
            'aui-dropdown2-hide.raven.testset .aui-dropdown2': '_onDownDownHide'
        },

        initialize: function() {
            var self = this;

            AJS.messages.setup();

            this._clearMessage();

            this._delegateUnscopedEvents();

            this.model = new RAVEN.TestSet.TestsViewModel();

            this.testSetKey = $('#raven-testset-tests-issue-key').val();

            this.canEdit = $('#can-edit-testset-test').val() === 'true';

            this._setEditable(this.canEdit);

            $('#raven-testset-tests').block();
            this.model.fetch({
                success: function() {
                    $('#raven-testset-tests').unblock();
                    if (self.$el.closest(document.documentElement).length > 0) {
                        self.render();
                    }
                },
                error: function(model, response) {
                    self.message(AJS.I18n.getText('view-issue-testset-tests-section.column-layout.error'), null, 'error');
                    $('#raven-testset-tests').unblock();
                }
            });
        },

        render: function() {
            this._clearMessage();

            // Render table skeleton
            var $table = RAVEN.TestSet.renderTestsTable({ 'columns': _.values(this.model.attributes) });
            var $tableWrapper = this.$el.find('#tests-table-wrapper');
            $tableWrapper.empty();
            $tableWrapper.html($table);

            // create the dataTable
            this._createTestsDataTable();

            // render bulk mode button
            if (this.canEdit === true && !this.bulkEditView) {
                this._renderBulkModeButton();
            }

            // render test runs toolbar view
            this._renderTestsToolbarView();

            $tableWrapper.show();

            this.$el.show();

            return this;
        },

        destroy: function() {
            this._cleanView();

            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();

            this.$el.removeData().unbind();

            this.unbind();

            this.remove();
        },

        _cleanView: function() {
            if (this.dataTable) {
                this.dataTable.fnDestroy(true);
                this.dataTable = null;
            }

            this.bulkEditView = null;
        },

        message: function(title, message, type) {
            type = (type === undefined || type == "") ? "error" : type;

            if (!message && title) {
                message = title;
                title = null;
            }

            if (type === "error") {
                this._clearMessage();
                AJS.messages.error('#raven-tests-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "success") {
                AJS.messages.success('#raven-tests-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "info") {
                AJS.messages.info('#raven-tests-message-bar', {
                    title: title,
                    body: message
                });
            }
        },

        _clearMessage: function() {
            $('#raven-tests-message-bar').empty();
        },

        _setEditable: function(editable) {
            if (editable == false) {
                $('#raven-add-test').hide();
                $('#raven-create-test').hide();
            } else {
                $('#raven-add-test').show();
                $('#raven-create-test').show();
            }
        },

        _createTestsDataTable: function() {
            var self = this;

            var path = contextPath + '/rest/raven/1.0/testset/' + this.testSetKey + "/tests";

            if (this.dataTable) {
                this.dataTable.fnDestroy();
                //$('#raven-testset-tests .tests-table').empty();
            }

            this.dataTable = $('#raven-testset-tests #testset-entries-table').xrayDataTable({
                "bProcessing": false,
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sEmptyTable": AJS.I18n.getText('view-issue-testset-tests-section.emptytable'),
                    "sInfoEmpty": AJS.I18n.getText('view-issue-testset-tests-section.infoempty'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch": AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "sDom": "l<t>ip",
                "bServerSide": true,
                "sAjaxSource": path,
                "sAjaxDataProp": "entries",
                "bProcessing": false,
                "bAutoWidth": false,
                "sPaginationType": "full_numbers",
                "bSort": false,
                "aaSorting": [],
                "aoColumnDefs": this._createDataTableColumnDefs(),
                "fnServerData": function(sSource, aoData, fnCallback, oSettings) {
                    self.$el.block();
                    oSettings.jqXHR = $.ajax({
                        "dataType": 'json',
                        "type": "GET",
                        "url": sSource,
                        "data": aoData,
                        "error": function(qXHR, textStatus, errorThrown) {
                            if (errorThrown && errorThrown.trim() != '') {
                                errorThrown = errorThrown + ': ' + qXHR.responseText;
                            } else {
                                errorThrown = qXHR.responseText;
                            }
                            self.message(AJS.I18n.getText('view-issue-testset-tests-section.processingerror'), errorThrown)
                        },
                        "complete": function() {
                            self.$el.unblock();
                        },
                        "success": fnCallback
                    });

                    if (self.bulkEditView) {
                        self.bulkEditView.setCheckBoxRowsSelectionValue(false);
                    }
                },
                "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    nRow.id = aData["key"];
                    return nRow;
                },
                "fnDrawCallback": function(oSettings) {
                    // set initial overflow to allow overflow columns to be visible
                    self.$el.css("overflow", "initial");

                    self._toogleTestsTable();
                    if (self.bulkEditView) {
                        self.bulkEditView.showHideBulkActions();
                    }
                }
            });

            this._initSortableColumns();

            if (this.canEdit === true) {
                this._initSortableTable(this.$el.find('#testset-entries-table'));
            }

        },

        /**
         * Creates the DataTable column def descriptors
         */
        _createDataTableColumnDefs: function() {
            var self = this;

            var columns = _.values(this.model.attributes);

            var bulkToogleColumnTitle = null;
            var bulkActionColumnTitle = null;
            if (self.canEdit === true) {
                bulkToogleColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkSelectAllInput();

                var bulkActions = '<a href="javascript:void(0);" data-key="bulkKey" class="bulk-remove-command active aui-dropdown2-active" title="' + AJS.I18n.getText('common.action.remove') + '">' +
                    '<span class="icon icon-delete menu-icon"></span>' + AJS.I18n.getText('common.action.remove') + '</a>';
                bulkActionColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkActionTrigger({ id: "tests-bulk-action", actions: bulkActions });
            }

            var firstColumnDefs = [{
                    "mData": null,
                    "sTitle": bulkToogleColumnTitle,
                    "mDataProp": null,
                    "sWidth": "20px",
                    "sDefaultContent": "<input type='checkbox' ></input>",
                    "bSortable": false,
                    "bSearchable": false,
                    "mRender": function(data, type, row) {
                        if (self.canEdit === true) {
                            return RAVEN.DataTable.BulkEdit.renderBulkSelectInputAndDragHandle({ id: row["key"] });
                        } else {
                            return '';
                        }
                    },
                    "aTargets": [0]
                },
                {
                    "mData": "rank",
                    "bSortable": false,
                    "bSearchable": false,
                    "mRender": function(data, type, row) {
                        return "<span class='rank' style='padding-left: 2px'>" + data + "</span>";
                    },
                    "aTargets": [1]
                }
            ];

            var userColumnDefs = this._createDataTableUserColumn(columns);

            var lastColumnDefs = [{
                "mData": null,
                "sTitle": bulkActionColumnTitle,
                "bSortable": false,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    if (self.canEdit === true) {
                        return RAVEN.TestSet.testsMenu({
                            key: row["key"],
                            canEdit: self.canEdit,
                            rank: row["rank"]
                        });
                    }
                    return null;
                },
                "aTargets": [2 + columns.length]
            }];

            return firstColumnDefs.concat(userColumnDefs).concat(lastColumnDefs);
        },

        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumn: function(columns) {
            return _.map(columns, function(column, index) {
                return {
                    "mData": 'userColumns.' + column.id,
                    "bSortable": false,
                    "bSearchable": false,
                    "aTargets": [2 + index]
                };
            });
        },


        _initSortableColumns: function() {
            var self = this;

            $('#testset-entries-table thead tr').sortable({
                items: 'th:not(.fixed)',
                placeholder: "dragging-placeholder",
                update: function(event, ui) {
                    var ancorNode = ui.item.prev();
                    var columnId = $(ui.item).data('column-id');
                    if (columnId) {
                        var anchorColumnId = null;
                        if (!$(ancorNode).hasClass('fixed')) {
                            anchorColumnId = $(ancorNode).data('column-id');
                        }

                        self.columnLayoutView.moveColumn(columnId, anchorColumnId);
                    }
                }
            });
        },

        _renderTestsToolbarView: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#toolbar-wrapper').length > 0) {
                return;
            }

            var toolbarHtml = RAVEN.TestSet.renderTestsToolBar();
            $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

            this.columnLayoutView = new RAVEN.TestSet.TestsColumnLayoutView({
                mainView: this
            });
        },

        /**
         * Adds support for moving datatable items
         */
        _initSortableTable: function(tableSelector) {
            var self = this;

            $(tableSelector).sortable({
                items: 'tbody tr',
                handle: '.draghandle',
                placeholder: "drag-placeholder",
                helper: function(e, tr) {
                    var $originals = tr.children();
                    var $helper = tr.clone();
                    $helper.children().each(function(index) {
                        // Set helper cell sizes to match the original sizes
                        $(this).width($originals.eq(index).width())
                    });
                    return $helper;
                },
                update: function(event, ui) {
                    var position = null;
                    var ancorNode = null;

                    var ancorNode = ui.item.prev();
                    if (ancorNode.length > 0) {
                        position = AFTER;
                    } else {
                        var ancorNode = ui.item.next();
                        if (ancorNode.length > 0) {
                            position = BEFORE;
                        }
                    }

                    if (position != null) {
                        var movedTestData = self.dataTable.fnGetData(ui.item[0]);
                        var ancorTestData = self.dataTable.fnGetData(ancorNode[0]);

                        if (movedTestData && movedTestData.key && ancorTestData && ancorTestData.key) {
                            self._moveTest(self.dataTable, movedTestData, ancorTestData, position);
                        }
                    }
                }
            });
        },

        _toogleTestsTable: function() {
            if (this.dataTable && this.dataTable.fnRecordsTotal() > 0) {
                this.$el.find('.notests').hide();
                this.$el.find('.xrayDataTables_wrapper').show();
            } else {
                this.$el.find('.xrayDataTables_wrapper').hide();
                this.$el.find('.notests').show();
            }
        },

        _renderBulkModeButton: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#bulk-edit-action').length > 0) {
                return;
            }

            this.bulkEditView = new RAVEN.DataTable.BulkEditView({
                el: $wrapper
            });
        },

        _updateRankRow: function(newRankValue, originalRankSpanValue, row) {
            //Validate values
            if (newRankValue > 0 && newRankValue <= this.dataTable.fnSettings().fnRecordsTotal() && newRankValue != originalRankSpanValue) {
                var movedTestData = this.dataTable.fnGetData(row);

                if (movedTestData && movedTestData.key) {
                    this._moveTest(this.dataTable, movedTestData, null, newRankValue);
                }
            }
        },

        _moveTest: function(table, movedTestData, ancorTestData, position) {
            var self = this;

            this.$el.block();

            var path = contextPath + '/rest/raven/1.0/testset/' + this.testSetKey + '/test/' + movedTestData.key + '/move'

            var value = {};
            if (position == AFTER) {
                value.after = '/' + ancorTestData.key;
            } else if (position == BEFORE) {
                value.before = '/' + ancorTestData.key;
            } else {
                value.index = position;
            }
            var data = JSON.stringify(value);

            $.ajax({
                type: "POST",
                url: path,
                processData: true,
                data: data,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('view-issue-testset-tests-section.move.error', movedTestData.key), errorThrown)
                },
                success: function(data, textStatus, jqXHR) {
                    self.dataTable.fnDraw(false);
                },
                complete: function() {
                    self.$el.unblock();
                }
            });
        },

        _addTests: function(testKeys) {
            var self = this;

            this.$el.block();

            var value = JSON.stringify(testKeys);
            $.ajax({
                type: "POST",
                url: contextPath + '/rest/raven/1.0/testset/' + this.testSetKey + '/tests',
                processData: true,
                async: true,
                data: value,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('view-issue-testset-tests-section.add.error', self.testSetKey), errorThrown)
                },
                success: function(data, textStatus, jqXHR) {
                    if (data.errors && data.errors.length > 0) {
                        var errorMsg = '';
                        $.each($(data.errors), function(i, msg) {
                            errorMsg = errorMsg + msg;
                            if (i < data.errors.length) {
                                errorMsg = errorMsg + '<br/>';
                            }
                        });
                        self.message(AJS.I18n.getText('view-issue-testset-tests-section.add.controlled.error', self.testSetKey), errorMsg);
                    }

                    if (data.tests && data.tests.length > 0) {
                        self.dataTable.fnAddData(data.tests);
                        // JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                    }
                },
                complete: function() {
                    self.$el.unblock();
                    self._updateHistory();
                    self._updateDates();
                }
            });
        },

        _confirmationDeleteTest: function(testKeys, yesFunction, bulk) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                id: 'delete-test-dialog',
                content: function(callback) {
                    var title = null;
                    var message = null;
                    if (bulk === true) {
                        title = AJS.I18n.getText('view-issue-testset-tests-section.bulk.delete.confirm.title');
                        message = AJS.I18n.getText('view-issue-testset-tests-section.bulk.delete.confirm.message', self.testSetKey);
                    } else {
                        title = AJS.I18n.getText('view-issue-testset-tests-section.delete.confirm.title', testKeys[0]);
                        message = AJS.I18n.getText('view-issue-testset-tests-section.delete.confirm.message', self.testSetKey);
                    }
                    var severity = "info";
                    var innerHtml = RAVEN.Dialog.deleteConfirmationDialog({ title: title, message: message, severity: severity });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                    e.preventDefault();
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(testKeys);
                    dialog.hide();
                },
                autoClose: true
            });
            dialog.show();
        },

        _deleteTest: function(testKeys) {
            var self = this;

            this.$el.block();

            //if we have records to delete
            if (testKeys && testKeys.length > 0) {
                //Build query param
                var testKeysParam = self._buildQueryParamsFromSelectedRows(testKeys);

                $.ajax({
                    type: "DELETE",
                    url: contextPath + '/rest/raven/1.0/testset/' + self.testSetKey + "/tests?" + testKeysParam + "",
                    processData: true,
                    async: true,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }
                        if (testKeys.length == 1) {
                            self.message(AJS.I18n.getText('view-issue-testset-tests-section.delete.error', testKeys[0]), errorThrown);
                        } else {
                            self.message(AJS.I18n.getText('view-issue-testset-tests-section.bulk.delete.error'), errorThrown);
                        }

                        self.$el.unblock();
                    },
                    success: function(data, textStatus, jqXHR) {
                        self._deleteTestsCallback(testKeys, data);
                    },
                    complete: function() {
                        self.$el.unblock();
                        self._updateHistory();
                        self._updateDates();
                    }
                });
            }
        },

        /**
         * Build query param
         */
        _buildQueryParamsFromSelectedRows: function(selectedIds) {
            var testKeys = "";
            var i;
            for (i = 0; i < selectedIds.length; ++i) {
                if (testKeys.length > 0) {
                    testKeys += "&";
                }
                testKeys += "testKeys=" + selectedIds[i];
            }

            return testKeys;
        },

        _deleteTestsCallback: function(testKeys, data) {
            var self = this;
            if (testKeys && testKeys.length > 0) {
                var allData = this.dataTable.fnGetData();
                if (allData.length >= 1) {
                    _.each(_.initial(testKeys), function(testKey) {
                        self.dataTable.fnDeleteRow(testKey, null, false);
                    });
                    this.dataTable.fnDeleteRow(_.last(testKeys));
                } else {
                    // don't refresh the table if the last element is removed because it will request more data from server
                    this.dataTable.fnDeleteRow(_.last(testKeys), null, false);
                }
            }
        },

        _confirmationChangeRank: function(title, message, row, originalRankSpanValue, yesFunction) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                content: function(callback) {
                    var innerHtml = RAVEN.Dialog.editValueDialog({ title: title, fieldName: message, actionText: AJS.I18n.getText('common.action.ok') });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                    e.preventDefault();
                    var newRankValue = $('#edit-value-dialog-value')[0].value;
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(newRankValue, originalRankSpanValue, row);
                    dialog.hide();
                },
                autoClose: true
            });
            dialog.show();
        },

        _createCreateTestForm: function() {
            var self = this;

            if (JIRA.Forms && JIRA.Forms.createCreateIssueForm) {
                var createTestForm = JIRA.Forms.createCreateIssueForm({
                    issueType: $('#test-issuetype-id').val()
                });

                var customFieldId = $('#testsets-customfield-id').val();

                // select trigger for create issue in global context
                var createTestDialog = createTestForm.asDialog({
                    trigger: document.createElement("a"), // Because we're showing it manually, we don't want to have the dialog re-show for a different dialog.
                    id: 'create-test-issue-dialog'
                });

                createTestDialog.bind('Dialog.show', function(e) {
                    window.createTestForTestSet = {
                        testSetsCustomFieldId: customFieldId,
                        testSetKey: self.testSetKey
                    };
                });

                createTestDialog.bind('Dialog.hide', function(evt, elem, reason, id) {
                    window.createTestForTestSet = null;

                    // we must always refresh the view because the reason might be 'cancel' and test issues are created
                    // prevent refresh issue page if dialog is stacked
                    if (!this.stacked) {
                        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                    }
                });

                createTestDialog.show();
            }
        },

        _updateHistory: function() {
            if ($('#changehistory-tabpanel').attr('class') == 'active') {
                var alt_token = $('input[name="atl_token"]').val();
                $.ajax({
                    type: "GET",
                    url: contextPath + '/browse/' + this.testSetKey + "?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_=" + alt_token,
                    contentType: "text/plain",
                    success: function(data, status) {
                        var tab = $(data).find('#activitymodule div.mod-content #issue_actions_container');
                        var jiraTab = $('#activitymodule div.mod-content').find('#issue_actions_container');
                        $(jiraTab).html(tab);
                    }
                });
            }
        },

        _updateDates: function() {
            var updatedDate = null;

            if (AJS.$('#datesmodule #updated-date').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-date time.livestamp');
            } else if (AJS.$('#datesmodule #updated-val').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-val time.livestamp');
            } else {
                return null;
            }

            $.ajax({
                type: "GET",
                url: AJS.contextPath() + '/rest/raven/1.0/date/' + JIRA.Issue.getIssueKey() + "/updated",
                contentType: "application/json",
                success: function(data) {
                    var title = data.title;
                    var dateTimeIso8601 = data.dateTimeIso8601;
                    updatedDate.attr("datetime", dateTimeIso8601);
                    $.each(AJS.$('#datesmodule dd.date'), function() {
                        if ($(this).find('#updated-date time.livestamp').length > 0 || $(this).find('#updated-val time.livestamp').length > 0) {
                            $(this).attr("title", title);
                        }
                    });
                    updatedDate.livestamp();
                }
            });
        },

        /**
         * Redraws the datatable
         */
        refresh: function() {
            this._cleanView();
            this.initialize();
        },


        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.testset');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.testset');
        },

        /**
         * Delete test click handler
         */
        _onTestDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var testKey = $(e.target).closest('a').data('key');
            this._confirmationDeleteTest([testKey], this._deleteTest);
        },

        /**
         * Delete Bulk test click handler
         */
        _onBulkTestDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var rowIds = this.bulkEditView.getSelectedRowIds();

            if (rowIds && rowIds.length > 0) {
                this._confirmationDeleteTest(rowIds, this._deleteTest, true);
            }
        },

        /**
         * Add tests button click handler
         */
        _onAddTestsClick: function() {
            var self = this;

            this._clearMessage();

            var labels = {};
            labels.label = AJS.I18n.getText('dialogs.issueselector.test.label');
            labels.title = AJS.I18n.getText('dialogs.issueselector.test.title', this.testSetKey);
            labels.description = AJS.I18n.getText('dialogs.issueselector.test.description');
            labels.enter = AJS.I18n.getText('dialogs.issueselector.test.enter');

            var issueTypeId = $('#test-issuetype-id').val();

            new RAVEN.Dialogs.createIssueSelectorAdvancedDialog('TEST', 'key not in testSetTests(' + this.testSetKey + ')', issueTypeId, this.testSetKey, labels, function(issues) {
                if (issues && issues.length > 0) {
                    self._addTests(issues);
                }
            }, null);
        },

        /**
         * Create Test button click handler
         */
        _onCreateTestClick: function(e) {
            this._clearMessage();
            this._createCreateTestForm();
            e.preventDefault();
        },

        /**
         * Change Test rank click handler.
         */
        _onChangeTestRankClick: function(e) {
            e.stopPropagation();
            e.preventDefault();
            this._clearMessage();

            var originalRankSpanValue = $(e.target).attr('data-key');

            var $row = $(this.currentDropdownTrigger).closest('tr');

            var rowKey = this.dataTable.fnGetData($row[0]).key;
            var label = AJS.I18n.getText('dialogs.edit-value.rank.label');
            var title = rowKey + ' - ' + AJS.I18n.getText('dialogs.edit-value.rank.title');
            this._confirmationChangeRank(title, label, $row[0], originalRankSpanValue, this._updateRankRow);
        },

        /**
         * Save current dropdown trigger
         */
        _onDropDownTriggerClick: function(e) {
            this.currentDropdownTrigger = $(e.target);
        },

        /**
         * Clear current dropdown trigger
         */
        _onDownDownHide: function() {}
    });

    RAVEN.TestSet.TestsView.REFRESH_VIEW = "refreshTestsView";
})(AJS.$);