(function ($) {

    window.RAVEN = window.RAVEN || {};

    var MAX_CONTENT_HEIGHT = 200;

    /**
     * An abstract class that gives the required behaviour for RestfulTable rows.
     * Extend this class and pass it as the {views.row} property of the options passed to RAVEN.RestfulTable in construction.
     *
     * @class Row
     * @namespace RAVEN.RestfulTable
     */
    RAVEN.RestfulTable.Row = Backbone.View.extend({

        // Static Const
        tagName: "tr",

        // delegate events
        events: {
            "click .raven-restfultable-editable" : "edit"
        },

        /**
         * @constructor
         * @param {object} options
         */
        initialize: function (options) {

            var instance = this;

            options = options || {};

            // faster lookup
            this._event = RAVEN.RestfulTable.Events;
            this.classNames = RAVEN.RestfulTable.ClassNames;
            this.dataKeys = RAVEN.RestfulTable.DataKeys;

            this.columns = options.columns;
            this.allowEdit = options.allowEdit;

            this.allowClone = options.allowClone;
            this.cloneCallback = options.cloneCallback;

            if (!this.events["click .raven-restfultable-editable"]) {
                throw new Error("It appears you have overridden the events property. To add events you will need to use"
                        + "a work around. https://github.com/documentcloud/backbone/issues/244")
            }
            this.index = options.index || 0;
            this.deleteConfirmation = options.deleteConfirmation;
            this.cloneConfirmation = options.cloneConfirmation;
            this.allowReorder = options.allowReorder;

            this.$el = $(this.el);

            this.bind(this._event.CANCEL, function () {
                this.disabled = true;
            })
            .bind(this._event.FOCUS, function (field) {
                this.focus(field);
            })
            .bind(this._event.BLUR, function () {
                this.unfocus();
            })
            .bind(this._event.UPDATED, function () {
                this._showUpdated();
            })
            .bind(this._event.MODAL, function () {
                this.$el.addClass(this.classNames.ACTIVE);
            })
            .bind(this._event.MODELESS, function () {
                this.$el.removeClass(this.classNames.ACTIVE)
            })
            .bind(this._event.ROW_APPENDED_TO_DOM, function () {
                // init row copy past
                this._ellipsize();
                //this.initRowCopyPaste();
            });
        },

        /**
         * Renders drag handle
         * @return jQuery
         */
        renderDragHandle: function () {
            return '<span class="' + this.classNames.DRAG_HANDLE + '"></span></td>';

        },

        /**
         * Renders default cell contents
         *
         * @param data
         * @return {undefiend, String}
         */
        defaultColumnRenderer: function (data) {
            if (data.value) {
                return document.createTextNode(data.value.toString());
            }
        },

        /**
         * Fades row from blue to transparent
         */
        _showUpdated: function () {

            var instance = this,
                    cells = this.$el
                            .addClass(this.classNames.ANIMATING)
                            .find("td")
                            .css("backgroundColor","#ebf1fd");

            this.trigger(this._event.ANIMATION_STARTED);

            instance.delegateEvents({});

            setTimeout(function () {
                cells.animate({
                    backgroundColor: "white"
                }, function () {
                    cells.css("backgroundColor", "");
                    instance.trigger(instance._event.ANIMATION_FINISHED);
                    $(document).one("mousemove", function () {
                        instance.delegateEvents();
                        instance.$el.removeClass(instance.classNames.ANIMATING);
                    });
                });
            }, 500)
        },

        /**
         * Save changed attributes back to server and re-render
         *
         * @param attr
         * @return {RAVEN.RestfulTable.Row}
         */
        sync: function (attr) {

            this.model.addExpand(attr);

            var instance = this;

            this.showLoading();

            this.model.save(attr, {
                success: function () {
                    instance.hideLoading().render();
                    instance.trigger(instance._event.UPDATED);
                },
                error: function () {
                    instance.hideLoading();
                }
            });

            return this;
        },

        /**
         * Get model from server and re-render
         *
         * @return {RAVEN.RestfulTable.Row}
         */
        refresh: function (success, error) {

            var instance = this;

            this.showLoading();

            this.model.fetch({
                success: function () {
                    instance.hideLoading().render();
                    if (success) {
                        success.apply(this, arguments);
                    }
                },
                error: function () {
                    instance.hideLoading();
                    if (error) {
                        error.apply(this, arguments);
                    }
                }
            });

            return this;
        },

        /**
         * Returns true if row has focused class
         *
         * @return Boolean
         */
        hasFocus: function () {
            return this.$el.hasClass(this.classNames.FOCUSED);
        },

        /**
         * Adds focus class (Item has been recently updated)
         *
         * @return RAVEN.RestfulTable.Row
         */
        focus: function () {
            $(this.el).addClass(this.classNames.FOCUSED);
            return this;
        },

        /**
         * Removes focus class
         *
         * @return RAVEN.RestfulTable.Row
         */
        unfocus: function () {
            $(this.el).removeClass(this.classNames.FOCUSED);
            return this;

        },

        /**
         * Adds loading class (to show server activity)
         *
         * @return RAVEN.RestfulTable.Row
         */
        showLoading: function () {
            this.$el.addClass(this.classNames.LOADING);
            return this;
        },

        /**
         * Hides loading class (to show server activity)
         *
         * @return RAVEN.RestfulTable.Row
         */
        hideLoading: function () {
            this.$el.removeClass(this.classNames.LOADING);
            return this;
        },

        /**
         * Switches row into edit mode
         *
         * @param e
         */
        edit: function (e) {
            var field;
            if ($(e.target).is("." + this.classNames.EDITABLE)) {
                field = $(e.target).attr("data-field-name");
            } else {
                field = $(e.target).closest("." + this.classNames.EDITABLE).attr("data-field-name");
            }
            this.trigger(this._event.ROW_EDIT, field);

            AJS.$("."+this.classNames.DRAG_HANDLE).hide();
            return this;
        },

        /**
         * Can be overriden to add custom options
         *
         */
        renderOperations: function () {
            var instance = this;

            if (this.allowEdit !== false) {
                var _dropdown = $('<div id="step-actions_' + instance.model.attributes.id + '" class="aui-dropdown2 aui-style-default"></div>');

                if(this.allowClone === true ){
                    var _cloneButton = $('<div class="aui-dropdown2-section"><ul><li><a title="' + AJS.I18n.getText('restful.table.action.clone') + '"><span class="icon icon-clone" style="padding-right: 3px;"></span>' + AJS.I18n.getText('restful.table.action.clone') + '</a></li></ul></div>')
                        .addClass(this.classNames.CLONE)
                        .click(function (e) {
                            e.preventDefault();
                            instance.cloneEntry();
                        });

                    _dropdown.append(_cloneButton);

                }

                var _moveToButton =  $('<div class="aui-dropdown2-section"><ul><li><a title="' + AJS.I18n.getText('common.action.move-to') + '"><span class="icon menu-icon aui-icon aui-icon-small aui-iconfont-switch-small"></span>' + AJS.I18n.getText('common.action.move-to') + '</a></li></ul></div>')
                    .click(function (e) {
                        e.preventDefault();
                        instance.changeRowRank();
                    });
                var _deleteButton =  $('<div class="aui-dropdown2-section"><ul><li><a title="' + AJS.I18n.getText("delete") + '"><span class="icon menu-icon icon-delete"></span>' + AJS.I18n.getText("delete") + '</a></li></ul></div>')
                    .addClass(this.classNames.DELETE)
                    .click(function (e) {
                        e.preventDefault();
                        instance.destroy();
                    });

                _dropdown.append(_moveToButton).append(_deleteButton);

                return $('<div aria-owns="step-actions_' + instance.model.attributes.id + '" title="' + AJS.I18n.getText('view-issue-testdetails.actions') + '"' +
                    'class="raven-dropdown-trigger aui-dropdown2-trigger aui-dropdown2-trigger-arrowless aui-icon aui-icon-small aui-iconfont-more actions-icon" ' +
                    'style="padding: 3px 0;" data-dropdown2-alignment="right"></div>').append(_dropdown);
            }
        },

        /**
         * Removes entry from table
         */
        destroy: function () {
            if (this.deleteConfirmation) {
				var self = this;
				var dialog = new JIRA.FormDialog({
					id: "delete-entity-" + this.model.get("id"),
					content: function(callback) {
						callback(self.deleteConfirmation(self.model));
					},
					submitHandler: function (e) {
						dialog.hide();
						self.model.destroy();
						e.preventDefault();
					},
					autoClose : false
				});

				dialog.show();
            } else {
                this.model.destroy();
            }
        },

        changeRowRank: function() {
            var stepId = $(this.el).attr('data-id');
            var originalRankValue = $(this.el).attr('data-index');

            var title = AJS.I18n.getText('dialogs.edit-value.step-rank.title', originalRankValue);
            var label = AJS.I18n.getText('dialogs.edit-value.rank.label');
            this.confirmationChangeRank(this, title, label, stepId, originalRankValue, this.updateRankRow);
        },

        confirmationChangeRank: function(row, title, message, stepId, originalRankValue, yesFunction) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                content: function (callback) {
                    var innerHtml = RAVEN.Dialog.editValueDialog({title:title, fieldName:message, actionText:AJS.I18n.getText('common.action.ok')});
                    callback(innerHtml);
                },
                submitHandler: function (e, callback) {
                    e.preventDefault();
                    var newRankValue = $('#edit-value-dialog-value')[0].value;
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(row, $('#raven-test-details-issue-key').val(), stepId, newRankValue, originalRankValue);
                    dialog.hide();
                },
                autoClose : true
            });

            $(dialog.get$popup()).on('before-submit', function(e) {
                var newRankValue = $('#edit-value-dialog-value')[0].value;

                if(isNaN(newRankValue) || !(newRankValue > 0 && newRankValue <= $(self.el).parent().children().length)) {
                    $('#dialog-invalid-value').text(AJS.I18n.getText('common.error.invalid-position'));
                    $('#dialog-invalid-value').show();
                    e.preventDefault();
                }
            });

            dialog.show();
        },

        updateRankRow: function(row, issueKey, stepId, newRankValue, originalRankValue) {
            //Validate values
            if(newRankValue > 0 && newRankValue <= $(this.el).parent().children().length && newRankValue != originalRankValue) {
                this.moveTestStep(row, issueKey, stepId, newRankValue);
            }
        },

        moveTestStep: function(row, issueKey, stepId, position) {
            var self = this;

            this.$el.block();

            var path = contextPath + '/rest/raven/1.0/test/' + issueKey + '/steps/' + stepId + '/move';

            var data = {};
            data.index = position;

            $.ajax({
                url: path,
                type: "POST",
                dataType: "json",
                contentType: "application/json",
                data: JSON.stringify(data),
                success: function(data, textStatus, jqXHR) {
                    self.trigger(self._event.ROW_MOVED, row, position);
                },
                complete: function() {
                    self.$el.unblock();
                }
            });
        },

        /**
         * Clones entry
         */
        cloneEntry: function () {
            var self = this;
            if (this.cloneConfirmation) {
                var self = this;
                var dialog = new JIRA.FormDialog({
                    id: "clone-entity-" + this.model.get("id"),
                    content: function(callback) {
                        callback(self.cloneConfirmation(self.model));
                    },
                    submitHandler: function (e) {
                        dialog.hide();
                        e.preventDefault(); 
                        self._innerCloneEntry(self);
                    },
                    autoClose : false
                });
                dialog.show();
            } else {
                _innerCloneEntry();
            }
        },

        _innerCloneEntry: function(self){
            $('span.raven-restful-table-row-operations').find('button').attr("disabled", true);
            $.ajax({
                type : "PUT",
                url : self.model.url()+'/clone',
                contentType : "application/json",
                success : function (data, status) {
                    self.cloneCallback(data);
                    $('span.raven-restful-table-row-operations').find('button').attr("disabled", false);
                }
            });
        },

        _createCopyDialog: function ($trigger, model, columnId, columnName, value) {
            var instance = this;

            var dialogId = 'copy-' + model.id + '-' + columnId;

            // remove previous dialog
            $('#inline-dialog-' + dialogId).remove();

            var $popup = AJS.InlineDialog($trigger, dialogId, function(content, trigger, showPopup) {
                // HACK: https://jira.atlassian.com/browse/JRA-24215
                $(content).click(function(e){e.stopPropagation()});

                var modelId = model.get("index");
                var heading = '#' + modelId + ' - ' + columnName + ' - ' + AJS.I18n.getText("restful.table.field.action.copy");

                var textareaId = 'copy-content-' + modelId + '-' + columnId;

                content.css({"padding":"10px"}).html('<h3>' + heading + '</h3>\
                    <form action="#" class="aui">\
                    <div id="inline-dialog-field-content">\
                    <textarea id="' + textareaId + '" class="textarea full-width-field" name="inline-' + columnId + '" style="resize: none;min-height: 200px;width: 100%;" readonly></textarea>\
                    </div>\
                    </form>');

                $("#"+textareaId).text(value);
                
                $(document).off('showLayer.raven').on("showLayer.raven", function(e, type, dialog) {
                    if (dialog && type === 'inlineDialog' && dialog.id === dialogId) {
                        var inlineDialog = document.getElementById('inline-dialog-' + dialogId);

                        // focus and select text area
                        var $textarea = $(inlineDialog).find('textarea');
                        if ($textarea.length > 0) {
                            $textarea.focus();

                            // IE HACK: Error when trying to select the textarea because the element is not visible yet
                            var ua = window.navigator.userAgent;
                            var msie = ua.indexOf("MSIE ");
                            if (msie > 0 || !!ua.match(/Trident.*rv\:11\./)) {
                                setTimeout(function() {
                                    try {
                                        document.getElementById(textareaId).select();
                                    } catch (err) {
                                        // ignore - nothing we can do
                                    }
                                }, 100);
                            } else {
                                $textarea.select();
                            }
                        }
                        if (!instance._isScrolledIntoView(inlineDialog)) {
                            var offset = $(inlineDialog).offset();
                            $('html, body').animate({
                                scrollTop: offset.top - 20,
                                scrollLeft: offset.left
                            });
                        }
                    }
                });

                showPopup();

                return false;
            },
            {
                // options
                width: 600,
                //onHover: true,
                //showDelay: 1000,
                isRelativeToMouse: true,
                cacheContent: false,
                hideDelay: null
            });
        },

        _ellipsize: function () {
            var instance = this;
             $.each(this.columns, function (i, column) {
                 if (column.allowEllipsize) {
                    var $contentWrapper = instance.$el.find('.' + instance.classNames.CONTENT);
                    $contentWrapper.each(function() {
                        var content = $(this)[0];
                        if ((content.offsetHeight + 1) < content.scrollHeight) { // IE scrollHeight() rounds the number of pixels up.
                            $(this).addClass('overflow');
                            var $showHiddenText = $(this).parent().parent().find('.cell-overflow-warning');
                            $showHiddenText.css('display', 'block');
                        }
                    });
                 }
             });
        },

        _initInlineDialog: function ($trigger, model, columnId, columnName, value) {
            var instance = this;

            var dialogId = 'view-' + model.id + '-' + columnId;

            // remove previous dialog
            $('#inline-dialog-' + dialogId).remove();

            var $popup = AJS.InlineDialog($trigger, dialogId, function(content, trigger, showPopup) {
                // HACK: https://jira.atlassian.com/browse/JRA-24215
                $(content).click(function(e){e.stopPropagation()});

                var modelId = model.get("index");

                var heading = '#' + modelId + ' - ' + columnName;

                var $fieldContent = $('<div class="inline-dialog-field-content" style="min-height: 200px">' + value + '</div>');

                content.css({"padding":"20px"}).html('<h3>' + heading + '</h3>');
                content.append($fieldContent);

                $(document).off('showLayer.raven').on("showLayer.raven", function(e, type, dialog) {
                    if (dialog && type === 'inlineDialog' && dialog.id === dialogId) {
                        var inlineDialog = document.getElementById('inline-dialog-' + dialogId);
                        if (!instance._isScrolledIntoView(inlineDialog)) {
                            var offset = $(inlineDialog).offset();
                            $('html, body').animate({
                                scrollTop: offset.top - 20,
                                scrollLeft: offset.left
                            });
                        }
                    }
                });

                showPopup();

                return false;
            },
            {
                // options
                width: 600,
                //onHover: true,
                //showDelay: 1000,
                isRelativeToMouse: true,
                cacheContent: false,
                hideDelay: null
            });
        },

        _isScrolledIntoView: function(elem) {
            var docViewTop = $(window).scrollTop();
            var docViewBottom = docViewTop + $(window).height();

            var offset = $(elem).offset();
            var elemTop = offset.top;
            var elemBottom = elemTop + $(elem).height();

            return ((elemBottom <= docViewBottom) && (elemTop >= docViewTop));
        },


        /**
         * Renders a generic edit row. You probably want to override this in a sub class.
         *
         * @return RAVEN.RestfulTable.Row
         */
        render: function  () {

            var instance = this,
                    renderData = this.model.toJSON(),
                    $opsCell = $("<td class='raven-restfultable-operations' />").append(this.renderOperations({}, renderData)),
                    $throbberCell = $("<td class='raven-restfultable-throbber' />");

            // restore state
            this.$el
                    .removeClass(this.classNames.DISABLED + " " + this.classNames.FOCUSED + " " + this.classNames.LOADING + " " + this.classNames.EDIT_ROW)
                    .addClass(this.classNames.READ_ONLY)
                    .empty();


            if (this.allowReorder) {
                $('<td  class="' + this.classNames.ORDER + '" />').append(this.renderDragHandle()).appendTo(instance.$el);
            }

            this.$el.attr("data-id", this.model.id); // helper for webdriver testing

            $.each(this.columns, function (i, column) {

                var contents,
                        $cell = $("<td />"),
                        value = renderData[column.id],
                        fieldName = column.fieldName || column.id,
                        args = [{name: fieldName, value: value, allowEdit: column.allowEdit}, renderData, instance.model];

                if (value) {
                    instance.$el.attr("data-" + column.id, value); // helper for webdriver testing

                }

				// RAVEN CHANGE: pass in the model when rendering a row
				var obj = args[0];
				obj.model = instance.model;
                if (column.readView) {
                    contents = new column.readView({
                        model: instance.model
                    }).render(obj);
                } else {
                    contents = instance.defaultColumnRenderer.apply(instance, args);
                }

                // wrap contents
                var $contentSection = null;
                if (column.allowEllipsize) {
                    $contentSection = $('<div />').addClass(instance.classNames.CONTENT).append(contents);
                } else {
                    $contentSection = $('<div />').append(contents);
                }

                if (instance.allowEdit !== false && column.allowEdit !== false) {
                    var $editableRegion = $("<span />")
                    		.addClass(instance.classNames.EDITABLE);
                    		//.append('<span class="icon icon-edit-sml" />');

                    if (column.allowCopy || column.InlineDialog) {
                        $editableRegion.addClass(instance.classNames.TOOLBAR);
                    } else {
                        $editableRegion.append('<span class="icon icon-edit-sml" />');
                    }

                    $editableRegion.append($contentSection).attr("data-field-name", fieldName);
                    
                    if (column.allowCopy) {
                        var random = new Date().valueOf();
                        copyPaste = column.allowCopy;

                        var $copyTrigger = $('<span class="icon icon-copy-sml" />')
                            .attr('title','Copy to Clipboard')
                            .attr('id',copyPaste.id+'_'+random);
                        
                        $editableRegion.append($copyTrigger);

                        instance._createCopyDialog($copyTrigger, instance.model, i, column.inlineDialog.name, column.allowCopy.getCopyValue(value));
                    }
                        
                    if (column.inlineDialog) {
                        var $trigger = $('<span class="icon icon-inline-dialog-sml" />').attr('title', 'click to expand view');
                        $editableRegion.append($trigger);

                        instance._initInlineDialog($trigger, instance.model, i, column.inlineDialog.name, contents);
                    }
                    
                    $cell  = $("<td />").append($editableRegion).appendTo(instance.$el);

                    if (column.allowEllipsize) {
                        var $contentWrapper = $cell.find('.' + instance.classNames.CONTENT);
                        $contentWrapper.each(function() {
                            var content = $(this)[0];
                                if ($cell.find('.cell-overflow-warning').length === 0) {
                                    var overflowWarning = "<span class='cell-overflow-warning'>" + AJS.I18n.getText('manual-test-steps-custom-field.cell-overflow-warning') + "</span>";
                                    $cell.append(overflowWarning);
                                    var $showHiddenText = $cell.find('.cell-overflow-warning');
                                    $showHiddenText.attr('title', 'click to expand view');
                                    $showHiddenText.css('display', 'none');
                                    instance._initInlineDialog($showHiddenText, instance.model, i, column.inlineDialog.name, contents);
                                }
                        });
                    }

                    if (!contents || $.trim(contents) == "") {
                        $cell.addClass(instance.classNames.NO_VALUE);
                        $editableRegion.html($('<span class="icon icon-edit-sml" />')).append($("<em />").text(this.emptyText || AJS.I18n.getText("aui.enter.value")));
                    }

                } else {
                    if (column.inlineDialog && !$contentSection.is(':empty')) {
                        var $focusableRegion = $("<span />").addClass(instance.classNames.FOCUSABLE).attr('title', 'click to expand view');
                        $focusableRegion.append($contentSection);
                        $cell.append($focusableRegion);

                        instance._initInlineDialog($focusableRegion, instance.model, i, column.inlineDialog.name, contents);
                    } else {
                        $cell.append($contentSection);
                    }
                }

                if (column.styleClass) {
                    $cell.addClass(column.styleClass);
                }

                $cell.appendTo(instance.$el);

                if (column.allowEllipsize) {
                    var $contentWrapper = $cell.find('.' + instance.classNames.CONTENT);
                    $contentWrapper.each(function() {
                        var content = $(this)[0];
                        if ((content.offsetHeight + 1) < content.scrollHeight) {
                            $(this).addClass('overflow');
                            if ($cell.find('.cell-overflow-warning').length === 0) {
                                var overflowWarning = "<span class='cell-overflow-warning'>" + AJS.I18n.getText('manual-test-steps-custom-field.cell-overflow-warning') + "</span>";
                                $cell.append(overflowWarning);
                            }
                            var $showHiddenText = $cell.find('.cell-overflow-warning');
                            $showHiddenText.attr('title', 'click to expand view');
                            $showHiddenText.css('display', 'block');
                            instance._initInlineDialog($showHiddenText, instance.model, i, column.inlineDialog.name, contents);
                        }
                    });
                }
            });

            this.$el
                    .append($opsCell)
                    .append($throbberCell)
                    .addClass(this.classNames.ROW + " " + this.classNames.READ_ONLY);

            this.trigger(this._event.RENDER, this.$el, renderData);
            this.$el.trigger(this._event.CONTENT_REFRESHED, [this.$el]);

            return this;
        }
    });

})(AJS.$);
